@@ -1,5 +1,4 @@
 import os
-
 os.environ["PYGAME_HIDE_SUPPORT_PROMPT"] = "hide"
 import curses
 from curses import textpad
@@ -8,6 +7,11 @@ import time
 import random
 import pickle
 from pathlib import Path
+from pytube import YouTube, Playlist
+import re
+import urllib.request
+import threading
+from pydub import AudioSegment
 
 RES_FOLDER = Path(__file__).parent / "res"
 QOUTE_FOLDER = Path(__file__).parent
@@ -39,6 +43,13 @@ def get_user_config_directory():
         return xdg_config_home
     return os.path.join(os.path.expanduser("~"), ".config")
 
+def isinternet():
+    try:
+        urllib.request.urlopen("https://youtube.com", timeout = 10) #Python 3.x
+        return True
+    except:
+        return False
+
 def replaceNth(
     s, source, target, n
 ):  # code from stack overflow, replaces nth occurence of an item.
@@ -111,7 +122,10 @@ def key_events(stdscr, tree1):
 
     if key == curses.KEY_ENTER or key == 10 or key == 13:  # this is enter key
         if tree1.showtimer:
-            tree1.starttimer(tree1.selectedtimer)
+            if tree1.currentmenu == "timer":
+                tree1.starttimer(tree1.selectedtimer)
+            else:
+                tree1.featureselect(tree1.selectedtimer)
             tree1.timerstart.play()
             tree1.showtimer = False
 
@@ -124,26 +138,38 @@ def key_events(stdscr, tree1):
         treedata = open(RES_FOLDER / "treedata", "wb")
         pickle.dump(tree1.age, treedata, protocol=None)
         treedata.close()
-
+        for file in list(_ for _ in QOUTE_FOLDER.glob("*.ogg")):
+            os.remove(file)
+        for file in list(_ for _ in QOUTE_FOLDER.glob("*.webm")):
+            os.remove(file)
         exit()
 
     if key in (curses.KEY_RIGHT, ord("l")):
-        tree1.music_list_num += 1
-        if tree1.music_list_num > len(tree1.music_list) - 1:
-            tree1.music_list_num = len(tree1.music_list) - 1
-        music = mixer.music.load(tree1.music_list[tree1.music_list_num])
-        mixer.music.play(-1)
-        tree1.show_music = True
-        tree1.musichidetime = int(time.time()) + 5
+        if tree1.showtimer:
+            tree1.selectedtimer = 0
+            tree1.currentmenu = "feature"
+
+        else:
+            tree1.music_list_num += 1
+            if tree1.music_list_num > len(tree1.music_list) - 1:
+                tree1.music_list_num = len(tree1.music_list) - 1
+            music = mixer.music.load(tree1.music_list[tree1.music_list_num])
+            mixer.music.play(-1)
+            tree1.show_music = True
+            tree1.musichidetime = int(time.time()) + 5
 
     if key in (curses.KEY_LEFT, ord("h")):
-        tree1.music_list_num -= 1
-        if tree1.music_list_num < 0:
-            tree1.music_list_num = 0
-        music = mixer.music.load(tree1.music_list[tree1.music_list_num])
-        mixer.music.play(-1)
-        tree1.show_music = True
-        tree1.musichidetime = int(time.time()) + 5
+        if tree1.showtimer:
+            tree1.selectedtimer = 0
+            tree1.currentmenu = "timer"
+        else:
+            tree1.music_list_num -= 1
+            if tree1.music_list_num < 0:
+                tree1.music_list_num = 0
+            music = mixer.music.load(tree1.music_list[tree1.music_list_num])
+            mixer.music.play(-1)
+            tree1.show_music = True
+            tree1.musichidetime = int(time.time()) + 5
 
     if key == ord(" "):
         mixer.music.pause()
@@ -157,6 +183,34 @@ def key_events(stdscr, tree1):
         else:
             mixer.music.unpause()
 
+    if key == ord("r"):
+        mixer.music.play()
+
+    if not tree1.isloading and key == ord("n"):
+        tree1.lofiradio()
+
+def GetSong(link):
+    songfile = str(YouTube("http://youtube.com/" + link.split("/")[-1] ).streams.get_by_itag(251).download())
+
+    AudioSegment.from_file(songfile, "webm").export(str(songfile + ".ogg"), format="ogg")
+
+    os.remove(songfile) 
+    if os.name == "posix":
+        songpath = str(songfile+".ogg").split("/")[-1]
+    else:
+        songpath = str(songfile+".ogg").split("\\")[-1]
+        
+    os.rename(songfile+".ogg", songpath)
+
+    return songpath
+
+def GetLinks(search_string):
+    html = urllib.request.urlopen("https://www.youtube.com/results?search_query=" + search_string.replace(" ", "+"))
+    video_ids = re.findall(r"watch\?v=(\S{11})", html.read().decode())
+    return "http://youtube.com/watch?v=" + str(video_ids[0])
+
+#print(GetSong(GetLinks(input())))
+#time.sleep(100)
 
 class tree:
     def __init__(self, stdscr, age):
@@ -169,12 +223,17 @@ class tree:
         self.pause = False
         self.showtimer = False
         self.timerlist = [
-            " Pomodro 20+20 ",
-            " Pomodro 20+10 ",
-            " Pomodro 40+20 ",
-            " Pomodro 50+10 ",
+            " POMODORO 20+20 ",
+            " POMODORO 20+10 ",
+            " POMODORO 40+20 ",
+            " POMODORO 50+10 ",
             " END TIMER NOW ",
         ]
+        self.featurelist = [
+            " PLAY MUSIC FROM YOUTUBE ",
+            " LOFI RADIO "
+        ]
+        self.currentmenu = "timer"
         self.selectedtimer = 0
         self.timerstart = mixer.Sound(str(RES_FOLDER / "timerstart.wav"))
         self.alarm = mixer.Sound(str(RES_FOLDER / "alarm.wav"))
@@ -187,6 +246,15 @@ class tree:
         self.season = random.choice(
             ["rain", "heavy_rain", "light_rain", "snow", "windy"]
         )
+        self.youtubedisplay = False
+        self.downloaddisplay = False
+        self.spinnerstate = 0
+        self.notifyendtime = 0
+        self.isnotify = False
+        self.notifystring = " "
+        self.playlist = Playlist("https://www.youtube.com/playlist?list=PL6fhs6TSspZvN45CPJApnMYVsWhkt55h7")
+        self.radiomode = False
+        self.isloading = False
         random.seed()
 
     def display(self, maxx, maxy, seconds):
@@ -250,30 +318,59 @@ class tree:
         if self.season == "windy":
             self.rain(maxx, maxy, seconds, 20, 30, "-", 4)
 
-    def timerdisplay(self, stdscr, maxy, maxx):
+    def notify(self, stdscr, maxy, maxx):
+        if self.isnotify and time.time() <= self.notifyendtime:
+            curses.textpad.rectangle(stdscr, 0,0,2, maxx-1)
+            stdscr.addstr(1,1, self.notifystring[:maxx-3-3] + "...", curses.A_BOLD)
+            self.downloaddisplay = False
+
+    def menudisplay(self, stdscr, maxy, maxx):
         if self.showtimer:
 
-            if self.selectedtimer > len(self.timerlist) - 1:
-                self.selectedtimer = len(self.timerlist) - 1
-            if self.selectedtimer < 0:
-                self.selectedtimer = 0
+            if self.currentmenu == "timer":
+                if self.selectedtimer > len(self.timerlist) - 1:
+                    self.selectedtimer = len(self.timerlist) - 1
+                if self.selectedtimer < 0:
+                    self.selectedtimer = 0
+
+            if self.currentmenu == "feature":
+                if self.selectedtimer > len(self.featurelist) - 1:
+                    self.selectedtimer = len(self.featurelist) - 1
+                if self.selectedtimer < 0:
+                    self.selectedtimer = 0
+
 
             for i in range(len(self.timerlist)):
-                if i == self.selectedtimer:
+                if i == self.selectedtimer and self.currentmenu == "timer":
                     stdscr.addstr(
-                        int((maxy - len(self.timerlist)) / 2) + i * 2,
-                        int(maxx / 10 - len(self.timerlist[i]) / 2) + 2,
+                        int((maxy - len(self.timerlist)*2) / 2) + i * 2,
+                        int(maxx / 25 + 4),
                         self.timerlist[i],
                         curses.A_REVERSE,
                     )
                 else:
                     stdscr.addstr(
-                        int((maxy - len(self.timerlist)) / 2) + i * 2,
-                        int(maxx / 10 - len(self.timerlist[i]) / 2),
+                        int((maxy - len(self.timerlist)*2) / 2) + i * 2,
+                        int(maxx / 25),
                         self.timerlist[i],
                     )
 
-        if int(time.time()) == self.timerhidetime:
+            for i in range(len(self.featurelist)):
+                if i == self.selectedtimer and self.currentmenu == "feature":
+                    stdscr.addstr(
+                        int((maxy - len(self.featurelist)*2) / 2) + i * 2,
+                        int(maxx * 24 / 25 - len(self.featurelist[i])) - 4,
+                        self.featurelist[i],
+                        curses.A_REVERSE,
+                    )
+                else:
+                    stdscr.addstr(
+                        int((maxy - len(self.featurelist)*2) / 2) + i * 2,
+                        int(maxx * 24 / 25 - len(self.featurelist[i])),
+                        self.featurelist[i],
+                    )
+
+        if int(time.time()) >= self.timerhidetime:
             self.showtimer = False
 
         if self.istimer:
@@ -329,7 +426,7 @@ class tree:
             self.breakstart()
 
     def starttimer(self, inputtime):
-        if inputtime >= len(TIMER_WORK):
+        if inputtime >= 4:
             self.worktime = 0
             self.breaktime = 0
             self.istimer == False
@@ -340,6 +437,172 @@ class tree:
 
         self.workendtime = int(time.time()) + self.worktime
 
+    def featureselect(self, inputfeature):
+        self.radiomode = False
+        if inputfeature == 0:
+            self.youtubedisplay = True
+        if inputfeature == 1:
+            self.playlist = Playlist("https://www.youtube.com/playlist?list=PLOzDu-MXXLliO9fBNZOQTBDddoA3FzZUo")
+            self.lofiradio()
+
+    def loading(self, stdscr, maxx):
+            spinner = [
+            "[    ]",
+            "[=   ]",
+            "[==  ]",
+            "[=== ]",
+            "[ ===]",
+            "[  ==]",
+            "[   =]",
+            "[    ]",
+            "[   =]",
+            "[  ==]",
+            "[ ===]",
+            "[====]",
+            "[=== ]",
+            "[==  ]",
+            "[=   ]"
+        ]
+            self.spinnerstate+=0.1
+            if self.spinnerstate > len(spinner)-1:
+                self.spinnerstate = 0
+            curses.textpad.rectangle(stdscr, 0,0,2, maxx-1)
+            stdscr.addstr(1,1, "GETTING AUDIO  " + spinner[int(self.spinnerstate)])
+
+
+
+    def youtube(self, stdscr, maxx):
+        if self.youtubedisplay:
+            curses.textpad.rectangle(stdscr, 0,0,2, maxx-1)
+            stdscr.addstr(1,1, "ENTER SEARCH QUERY/LINK : ")
+            stdscr.refresh()
+
+            if not "songinput" in locals():
+
+                curses.echo()
+                curses.nocbreak()
+                stdscr.nodelay(False)
+                stdscr.keypad(False)
+                curses.curs_set(1)
+
+                songinput = str(stdscr.getstr())
+
+                curses.noecho()
+                curses.cbreak()
+                stdscr.nodelay(True)
+                stdscr.keypad(True)
+                curses.curs_set(0)
+
+
+            stdscr.addstr(1,1, "GETTING AUDIO")
+
+            getsongthread = threading.Thread(target=self.playyoutube, args=(songinput,))
+            getsongthread.daemon = True
+            getsongthread.start()
+
+            self.youtubedisplay = False
+
+            self.downloaddisplay = True
+
+            del songinput
+
+
+
+        if self.downloaddisplay:
+            self.loading(stdscr, maxx)
+
+
+    def playyoutube(self, songinput):
+
+        try:
+            song = GetSong(GetLinks(songinput))
+
+            mixer.music.load(song)
+            mixer.music.play()
+
+            os.remove(song)
+
+        except:
+            self.notifyendtime = int(time.time()) + 5
+            self.isnotify = True
+            self.notifystring = "ERROR GETTING AUDIO, PLEASE TRY AGAIN"
+            for file in list(_ for _ in QOUTE_FOLDER.glob("*.ogg")):
+                os.remove(file)
+            for file in list(_ for _ in QOUTE_FOLDER.glob("*.webm")):
+                os.remove(file)
+            exit()
+
+        self.downloaddisplay = False
+
+
+        self.notifyendtime = int(time.time()) + 10
+        self.isnotify = True
+        self.notifystring = "Playing: " + str(song).split("/")[-1].split(".webm.ogg")[0]
+
+    def getlofisong(self): # some links dont work, usre recurtion to find a link which works
+        try:
+            song = GetSong(self.playlist[random.randrange(0, len(self.playlist))])
+        except:
+            if isinternet():
+                song = self.getlofisong()
+            else:
+                self.isloading = False
+
+                self.notifyendtime = int(time.time()) + 10
+                self.isnotify = True
+                self.notifystring = "UNABLE TO CONNECT"
+                self.radiomode = False
+                for file in list(_ for _ in QOUTE_FOLDER.glob("*.ogg")):
+                    os.remove(file)
+                for file in list(_ for _ in QOUTE_FOLDER.glob("*.webm")):
+                    os.remove(file)
+                exit()
+
+        return song
+
+    def lofiradio(self):
+         #lofi playlist from youtube
+
+        self.isloading = True
+        self.radiomode = True
+
+        radiothread = threading.Thread(target=self.actuallofiradio)
+        radiothread.daemon = True
+        radiothread.start()
+
+
+    def actuallofiradio(self):
+
+        try:
+
+            if not hasattr(self, "lofisong"):
+                self.lofisong = self.getlofisong()
+
+            mixer.music.load(self.lofisong)
+            mixer.music.play()
+
+            self.lofisonglen = mixer.Sound(self.lofisong).get_length()
+
+            self.notifyendtime = int(time.time()) + 10
+            self.isnotify = True
+            self.notifystring = "Playing: " + str(self.lofisong).split("/")[-1].split(".webm.ogg")[0]
+
+            os.remove(self.lofisong)
+            self.lofisong = self.getlofisong()
+
+            self.isloading = False
+
+        except:
+            self.isloading = False
+            self.radiomodeo = False
+
+            self.notifyendtime = int(time.time()) + 10
+            self.isnotify = True
+            self.notifystring = "ERROR GETTING AUDIO, PLEASE TRY AGAIN"
+            exit()
+
+
+
 
 def main():
     run = True
@@ -379,6 +642,7 @@ def main():
     treedata_in = open(RES_FOLDER/ "treedata", "rb")
     tree1.age = pickle.load(treedata_in)
 
+
     try:
         while run:
 
@@ -399,7 +663,7 @@ def main():
                     anilen = 1
                     tree_grow.play()
 
-                if tree1.musichidetime == int(time.time()):
+                if tree1.musichidetime <= int(time.time()):
                     tree1.show_music = False
 
                 if tree1.show_music:
@@ -428,10 +692,20 @@ def main():
 
                 tree1.seasons(maxx, maxy, seconds)
 
-                tree1.timerdisplay(stdscr, maxy, maxx)
+                tree1.menudisplay(stdscr, maxy, maxx)
+
+                tree1.youtube(stdscr, maxx)
 
                 tree1.timer()
 
+                if not tree1.isloading and tree1.radiomode and mixer.music.get_pos() == -1:
+                    tree1.lofiradio()
+
+                if tree1.isloading:
+                    tree1.loading(stdscr, maxx)
+
+                tree1.notify(stdscr, maxy, maxx)
+
                 mixer.music.set_volume(music_volume)
 
                 key_events(stdscr, tree1)
@@ -456,6 +730,10 @@ def main():
                         treedata = open(RES_FOLDER / "treedata", "wb")
                         pickle.dump(tree1.age, treedata, protocol=None)
                         treedata.close()
+                        for file in list(_ for _ in QOUTE_FOLDER.glob("*.ogg")):
+                            os.remove(file)
+                        for file in list(_ for _ in QOUTE_FOLDER.glob("*.webm")):
+                            os.remove(file)
                         exit()
 
                 while tree1.isbrake:
@@ -479,6 +757,10 @@ def main():
                         treedata = open(RES_FOLDER / "treedata", "wb")
                         pickle.dump(tree1.age, treedata, protocol=None)
                         treedata.close()
+                        for file in list(_ for _ in QOUTE_FOLDER.glob("*.ogg")):
+                            os.remove(file)
+                        for file in list(_ for _ in QOUTE_FOLDER.glob("*.webm")):
+                            os.remove(file)
                         exit()
 
                 time.sleep(0.01)
